c-----------------------------------------------------------------------
c     PRM_HH by A.V.Melnikov (2018)
c
c     The case of bounded phase space (Henon-Heiles system)
c     H = (p_x**2+p_y**2+q_x**2+q_y**2)/2+2*q_x**2*q_y-2/3*q_y**3
c     This code realizes the PRM algorithm.
c     The code contains calls to the dop853 integrator (external).
c-----------------------------------------------------------------------
      subroutine fcn(n,t,y,f)
c    the equations of motion 

      implicit none

      integer n
      real*8 t,y(n),f(n)
 
c     f(1) = dp_x/dt, f(2) = dp_y/dt, f(3) = dq_x/dt, f(4) = dq_y/dt
      f(1) = -y(3) - 2.d0*y(3)*y(4)
      f(2) = -y(4) - y(3)**2 + y(4)**2
      f(3) = y(1)
      f(4) = y(2)
      
      return
      end
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
      subroutine solout(nr, xold, x, y, n, con, icomp, nd,
     +                  rpar, ipar, irtrn)
c    check the distance from the current position of
c    the phase trajectory to the initial point of integration   

      implicit none

      integer nr, n, nd, ipar, irtrn
      real*8 xold, x, y(n), con(8*nd),rpar
      integer icomp(nd)

      real*8 point(4),eps,delta,rec_time

      common /inint_point/ point,eps,rec_time
  
      delta = dsqrt((y(1)-point(1))**2 + (y(2)-point(2))**2 +
     +                      (y(3)-point(3))**2 + (y(4)-point(4))**2)

c     determine if there is a recurrence at this integration step     
      if ((delta.le.eps).and.(x.gt.1.d0)) then
c     fixing the recurrence time and stopping the integration       
          rec_time = x
          irtrn = -1   
      end if

      return
      end
c-----------------------------------------------------------------------
